/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: Freescale MKV10Z32xxx7 LED Flasher
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2013 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/
#include "MKV10Z7.h"                    // Device header


#define LED_NUM     5                   /* Number of user LEDs                */
const uint32_t     led_num []   = {1, 25, 3, 4, 4};
      GPIO_Type *  gpio_adr[]   = {GPIOC, GPIOE, GPIOC, GPIOC, GPIOD};
      PORT_Type *  port_adr[]   = {PORTC, PORTE, PORTC, PORTC, PORTD};
volatile uint32_t msTicks;                            /* counts 1ms timeTicks */
/*----------------------------------------------------------------------------
  SysTick_Handler
 *----------------------------------------------------------------------------*/
void SysTick_Handler(void) {
  msTicks++;                        /* increment counter necessary in Delay() */
}

/*------------------------------------------------------------------------------
  delays number of tick Systicks (happens every 1 ms)
 *------------------------------------------------------------------------------*/
__INLINE static void Delay (uint32_t dlyTicks) {
  uint32_t curTicks;

  curTicks = msTicks;
  while ((msTicks - curTicks) < dlyTicks);
}

/*------------------------------------------------------------------------------
  configer LED pins
 *------------------------------------------------------------------------------*/
__INLINE static void LED_Config(void) {
  int i;
  SIM->SCGC5    |= (1UL <<  11) | (1UL <<  12) | (1UL <<  13);      /* Enable Clock to Port C,D,E */ 
  for (i = 0; i < LED_NUM; i++) 
  { 
    port_adr[i]->PCR[led_num[i]]   =   (1UL <<  8);
    gpio_adr[i]->PDOR             |=   1 << led_num[i];
    gpio_adr[i]->PDDR             |=   1 << led_num[i];
  }
}

/*------------------------------------------------------------------------------
  Switch on LEDs
 *------------------------------------------------------------------------------*/
__INLINE static void LED_On (uint32_t led) {
  gpio_adr[led]->PCOR  =   1 << led_num[led];
}


/*------------------------------------------------------------------------------
  Switch off LEDs
 *------------------------------------------------------------------------------*/
__INLINE static void LED_Off (uint32_t led) {
  gpio_adr[led]->PSOR   =  1 << led_num[led];
}
/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/

int main(void)
{
  int num     = -1; 
  int dir     =  1;

  SystemCoreClockUpdate();                      /* Get Core Clock Frequency */
  SysTick_Config(SystemCoreClock/1000);         /* Generate interrupt each 1 ms    */
  
  LED_Config();                             
 
  while(1) {
    /* Calculate 'num': 0,1,...,LED_NUM-1,LED_NUM-1,...,1,0,0,...             */
    num += dir;
    if (num == LED_NUM) { dir = -1; num =  LED_NUM-1; } 
    else if   (num < 0) { dir =  1; num =  0;         }
    LED_On (num);
    Delay(500);
    LED_Off(num);
  }
}
