/* -----------------------------------------------------------------------------
 * Copyright (c) 2016 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        29. February 2016
 * $Revision:    V1.00
 *
 * Project:      Flash Device Description for Kinetis KV3x
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */

#include "..\FlashOS.H"        /* FlashOS Structures */


#ifdef P64_2KBSec
struct FlashDevice const FlashDevice  =  {
    FLASH_DRV_VERS,                     /* Driver Version, do not modify! */
  #if   defined (KV3x)
    "MKV3x 64KB PFlash (2KB Sector)",   /* Device Name */
  #else
    "MKxxx 64kB PFlash (2KB Sector)",   /* Device Name */
  #endif
    ONCHIP,                             /* Device Type */
    0x00000000,                         /* Device Start Address */
    0x00010000,                         /* Device Size (64kB) */
    512,                                /* Programming Page Size */
    0,                                  /* Reserved, must be 0 */
    0xFF,                               /* Initial Content of Erased Memory */
    1000,                               /* Program Page Timeout 1000 mSec */
    3000,                               /* Erase Sector Timeout 3000 mSec */
    /* Specify Size and Address of Sectors */
    0x000800, 0x000000,                 /* Sector Size  2kB ( 32 Sectors) */
    SECTOR_END
};
#endif

#ifdef P128_2KBSec
struct FlashDevice const FlashDevice  =  {
    FLASH_DRV_VERS,                     /* Driver Version, do not modify! */
  #if   defined (KV3x)
    "MKV3x 128KB PFlash (2KB Sector)",  /* Device Name */
  #else
    "MKxxx 128kB PFlash (2KB Sector)",  /* Device Name */
  #endif
    ONCHIP,                             /* Device Type */
    0x00000000,                         /* Device Start Address */
    0x00020000,                         /* Device Size (128kB) */
    512,                                /* Programming Page Size */
    0,                                  /* Reserved, must be 0 */
    0xFF,                               /* Initial Content of Erased Memory */
    1000,                               /* Program Page Timeout 1000 mSec */
    3000,                               /* Erase Sector Timeout 3000 mSec */
    /* Specify Size and Address of Sectors */
    0x000800, 0x000000,                 /* Sector Size  2kB ( 64 Sectors) */
    SECTOR_END
};
#endif

#ifdef P256_2KBSec
struct FlashDevice const FlashDevice  =  {
    FLASH_DRV_VERS,                     /* Driver Version, do not modify! */
  #if   defined (KV3x)
    "MKV3x 256KB PFlash (2KB Sector)",  /* Device Name */
  #else
    "MKxxx 256kB PFlash (2KB Sector)",  /* Device Name  */
  #endif
    ONCHIP,                             /* Device Type */
    0x00000000,                         /* Device Start Address */
    0x00040000,                         /* Device Size (256kB) */
    512,                                /* Programming Page Size */
    0,                                  /* Reserved, must be 0 */
    0xFF,                               /* Initial Content of Erased Memory */
    1000,                               /* Program Page Timeout 1000 mSec */
    3000,                               /* Erase Sector Timeout 3000 mSec */

    /* Specify Size and Address of Sectors */
    0x000800, 0x000000,                 /* Sector Size  2kB (128 Sectors) */
    SECTOR_END
};
#endif

#ifdef P512_2KBSec
struct FlashDevice const FlashDevice  =  {
    FLASH_DRV_VERS,                     /* Driver Version, do not modify! */
  #if   defined (KV3x)
    "MKV3x 512KB PFlash (2KB Sector)",  /* Device Name */
  #else
    "MKxxx 512kB PFlash (2KB Sector)",  /* Device Name */
  #endif
    ONCHIP,                             /* Device Type */
    0x00000000,                         /* Device Start Address */
    0x00080000,                         /* Device Size (512kB) */
    512,                                /* Programming Page Size */
    0,                                  /* Reserved, must be 0 */
    0xFF,                               /* Initial Content of Erased Memory */
    1000,                               /* Program Page Timeout 1000 mSec */
    3000,                               /* Erase Sector Timeout 3000 mSec */
    /* Specify Size and Address of Sectors */
    0x000800, 0x000000,                 /* Sector Size  2kB (256 Sectors) */
    SECTOR_END
};
#endif



#ifdef FAC
struct FlashDevice const FlashDevice  =  {
    FLASH_DRV_VERS,                     /* Driver Version, do not modify! */
    "MKV3x  Flash Access Control",      /* Device Name */
    ONCHIP,                             /* Device Type */
    0xFFFF0000,                         /* Device Start Address (virtual address) */
    0x00000024,                         /* Device Size (4 * 8Bytes + 4Bytes) */
    36,                                 /* Programming Page Size */
    0,                                  /* Reserved, must be 0 */
    0xFF,                               /* Initial Content of Erased Memory */
    1000,                               /* Program Page Timeout 1000 mSec */
    3000,                               /* Erase Sector Timeout 3000 mSec */

    /* Specify Size and Address of Sectors */
    0x000024, 0x000000,                 /* Sector Size  36B (1 Sectors) */
    SECTOR_END
};
#endif
