/* -----------------------------------------------------------------------------
 * Copyright (c) 2014 - 2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *   
 *
 * $Date:        01. July 2015
 * $Revision:    V1.02
 *  
 * Project:      Flash Device Description for Freescale MKVxx Internal Flash
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.02
 *    Added MKV_P64_2KB_SEC, MKV_P128_2KB_SEC
 *  Version 1.01
 *    Added MKV_P512_8KB_SEC, MKV_P1024_8KB_SEC
 *  Version 1.00
 *    Initial release
 */ 

#include "..\FlashOS.H"        // FlashOS Structures


#ifdef MKV_P16_1KB_SEC
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,                  // Driver Version, do not modify!
  "MKVxx 16KB P-Flash(1KB Sector)", // Device Name 
   ONCHIP,                          // Device Type
   0x00000000,                      // Device Start Address
   0x00004000,                      // Device Size (16kB)
   256,                             // Programming Page Size
   0,                               // Reserved, must be 0
   0xFF,                            // Initial Content of Erased Memory
   1000,                            // Program Page Timeout 1000 mSec
   3000,                            // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x000400, 0x000000,              // Sector Size  1kB (16 Sectors)
   SECTOR_END
};
#endif

#ifdef MKV_P32_1KB_SEC
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,                  // Driver Version, do not modify!
  "MKVxx 32KB P-Flash(1KB Sector)", // Device Name 
   ONCHIP,                          // Device Type
   0x00000000,                      // Device Start Address
   0x00008000,                      // Device Size (32kB)
   256,                             // Programming Page Size
   0,                               // Reserved, must be 0
   0xFF,                            // Initial Content of Erased Memory
   1000,                            // Program Page Timeout 1000 mSec
   3000,                            // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x000400, 0x000000,              // Sector Size  1kB (32 Sectors)
   SECTOR_END
};
#endif

#ifdef MKV_P64_2KB_SEC
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,                  // Driver Version, do not modify!
  "MKVxx 64KB P-Flash(2KB Sector)", // Device Name 
   ONCHIP,                          // Device Type
   0x00000000,                      // Device Start Address
   0x00010000,                      // Device Size (64kB)
   256,                             // Programming Page Size
   0,                               // Reserved, must be 0
   0xFF,                            // Initial Content of Erased Memory
   1000,                            // Program Page Timeout 1000 mSec
   3000,                            // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x000800, 0x000000,              // Sector Size  2kB (32 Sectors)
   SECTOR_END
};
#endif

#ifdef MKV_P128_2KB_SEC
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,                  // Driver Version, do not modify!
  "MKVxx 128KB P-Flash(2KB Sector)", // Device Name 
   ONCHIP,                          // Device Type
   0x00000000,                      // Device Start Address
   0x00020000,                      // Device Size (128kB)
   256,                             // Programming Page Size
   0,                               // Reserved, must be 0
   0xFF,                            // Initial Content of Erased Memory
   1000,                            // Program Page Timeout 1000 mSec
   3000,                            // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x000800, 0x000000,              // Sector Size  2kB (64 Sectors)
   SECTOR_END
};
#endif

#ifdef MKV_P512_8KB_SEC
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,                  // Driver Version, do not modify!
  "MKVxx 512KB P-Flash(8KB Sector)", // Device Name 
   ONCHIP,                          // Device Type
   0x10000000,                      // Device Start Address
   0x00080000,                      // Device Size (512kB)
   512,                             // Programming Page Size
   0,                               // Reserved, must be 0
   0xFF,                            // Initial Content of Erased Memory
   1000,                            // Program Page Timeout 1000 mSec
   3000,                            // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x002000, 0x000000,              // Sector Size  8kB (64 Sectors)
   SECTOR_END
};
#endif

#ifdef MKV_P1024_8KB_SEC
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,                  // Driver Version, do not modify!
  "MKVxx 1024KB P-Flash(8KB Sector)", // Device Name 
   ONCHIP,                          // Device Type
   0x10000000,                      // Device Start Address
   0x00100000,                      // Device Size (512kB)
   512,                             // Programming Page Size
   0,                               // Reserved, must be 0
   0xFF,                            // Initial Content of Erased Memory
   1000,                            // Program Page Timeout 1000 mSec
   3000,                            // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x002000, 0x000000,              // Sector Size  8kB (64 Sectors)
   SECTOR_END
};
#endif


#ifdef MKV_PIFR_75MHZ
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,                  // Driver Version, do not modify!
   "MKVxx 75Mhz PIFR",              // Device Name 
   ONCHIP,                          // Device Type
   0x0FFFFFC0,                      // Device Start Address
   0x00000040,                      // Device Size (64 Byte)
   64,                              // Programming Page Size
   0,                               // Reserved, must be 0
   0xFF,                            // Initial Content of Erased Memory
   1000,                            // Program Page Timeout 1000 mSec
   3000,                            // Erase Sector Timeout 3000 mSec
// Specify Size and Address of Sectors
   0x000040, 0x000000,              // Sector Size  64 Byte (1 Sector)
   SECTOR_END
};
#endif
